function imgOut = LinPredCoarse(I,mask)
% Metodo il quale, data un'image in ingresso, predice (solo) la zona selezionata
% dalla maschera. Metodo da applicare al livello Coarse
% Predizione del livello COARSES.
%
% X X
% X O
%
% e 4 varianti.
%
% I = input intensity image
% mask = mashera
% imgOut = output image


[Height,Width,Depth] = size(I);
if Depth > 1
    I1 = double(rgb2gray(I));
else
    I1 = double(I);
end

I1 = I1 .* (~mask);

img_a = I1;
img_b = I1;
img_c = I1;
img_d = I1;


%% ------------------- Coefficienti -------------------

%Calcolo della autocorrelazione normalizzata
Cor = autoCorr2DNormalized(I);

% Calcolo dei coefficienti alfa_a

R_u_a = zeros(3,3);
R_u_a(1,1)=Cor(Height,Width);
R_u_a(1,2)=Cor(Height-1,Width+1);
R_u_a(1,3)=Cor(Height-1,Width);
R_u_a(2,1)=Cor(Height+1,Width-1);
R_u_a(2,2)=R_u_a(1,1);
R_u_a(2,3)=Cor(Height,Width-1);
R_u_a(3,1)=Cor(Height+1,Width);
R_u_a(3,2)=Cor(Height,Width+1);
R_u_a(3,3)=R_u_a(1,1);

r_u_a=zeros(3,1);
r_u_a(1)=Cor(Height,Width+1);
r_u_a(2)=Cor(Height+1,Width);
r_u_a(3)=Cor(Height+1,Width+1);

alfa_a=zeros(3,1);
alfa_a=R_u_a\r_u_a;


% Calcolo dei coefficienti alfa_b

R_u_b = zeros(3,3);
R_u_b(1,1)=Cor(Height,Width);
R_u_b(1,2)=Cor(Height-1,Width-1);
R_u_b(1,3)=Cor(Height-1,Width);
R_u_b(2,1)=Cor(Height+1,Width+1);
R_u_b(2,2)=R_u_b(1,1);
R_u_b(2,3)=Cor(Height,Width+1);
R_u_b(3,1)=Cor(Height+1,Width);
R_u_b(3,2)=Cor(Height,Width-1);
R_u_b(3,3)=R_u_b(1,1);

r_u_b=zeros(3,1);
r_u_b(1)=Cor(Height,Width-1);
r_u_b(2)=Cor(Height+1,Width);
r_u_b(3)=Cor(Height+1,Width-1);

alfa_b=zeros(3,1);
alfa_b=R_u_b\r_u_b;


% Calcolo dei coefficienti alfa_c

R_u_c = zeros(3,3);
R_u_c(1,1)=Cor(Height,Width);
R_u_c(1,2)=Cor(Height+1,Width+1);
R_u_c(1,3)=Cor(Height+1,Width);
R_u_c(2,1)=Cor(Height-1,Width-1);
R_u_c(2,2)=R_u_c(1,1);
R_u_c(2,3)=Cor(Height,Width-1);
R_u_c(3,1)=Cor(Height-1,Width);
R_u_c(3,2)=Cor(Height,Width+1);
R_u_c(3,3)=R_u_c(1,1);

r_u_c=zeros(3,1);
r_u_c(1)=Cor(Height,Width+1);
r_u_c(2)=Cor(Height-1,Width);
r_u_c(3)=Cor(Height-1,Width+1);

alfa_c=zeros(3,1);
alfa_c=R_u_c\r_u_c;


% Calcolo dei coefficienti alfa_d

R_u_d = zeros(3,3);
R_u_d(1,1)=Cor(Height,Width);
R_u_d(1,2)=Cor(Height+1,Width-1);
R_u_d(1,3)=Cor(Height+1,Width);
R_u_d(2,1)=Cor(Height-1,Width+1);
R_u_d(2,2)=R_u_d(1,1);
R_u_d(2,3)=Cor(Height,Width+1);
R_u_d(3,1)=Cor(Height-1,Width);
R_u_d(3,2)=Cor(Height,Width+1);
R_u_d(3,3)=R_u_d(1,1);

r_u_d=zeros(3,1);
r_u_d(1)=Cor(Height,Width-1);
r_u_d(2)=Cor(Height-1,Width);
r_u_d(3)=Cor(Height-1,Width-1);

alfa_d=zeros(3,1);
alfa_d=R_u_d\r_u_d;


%% ------------------- Predizione -------------------

% -- Predizione da alto->basso e sx->dx -- 

for r = 2:Height
    for c = 2:Width
        if(mask(r,c)>=1)
            img_a(r,c) = alfa_a(1)*img_a(r,c-1) + alfa_a(2)*img_a(r-1,c) + alfa_a(3)*img_a(r-1,c-1);              
        end
    end
end

% -- Predizione da alto->basso e dx->sx -- 

for r = 2:Height
    for c = Width-1:-1:1
        if(mask(r,c)>=1)
            img_b(r,c) = alfa_b(1)*img_b(r,c+1) + alfa_b(2)*img_b(r-1,c) + alfa_b(3)*img_b(r-1,c+1);              
        end
    end
end

% -- Predizione da basso->alto e sx->dx -- 

for r = Height-1:-1:1
    for c = 2:Width
        if(mask(r,c)>=1)
            img_c(r,c) = alfa_c(1)*img_c(r,c-1) + alfa_c(2)*img_c(r+1,c) + alfa_c(3)*img_c(r+1,c-1);              
        end
    end
end

% -- Predizione da basso->alto e dx->sx -- 

for r = Height-1:-1:1
    for c = Width-1:-1:1
        if(mask(r,c)>=1)
            img_d(r,c) = alfa_d(1)*img_d(r,c+1) + alfa_d(2)*img_d(r+1,c) + alfa_d(3)*img_d(r+1,c+1);              
        end
    end
end
   
    
%% ------------------- Baricentro e Maschere -------------------

M=mask;
[m,n]=size(M);
baricentro = [0,0];
totale=0;
for i=1:m
    for j=1:n
        if(M(i,j)==1)
            totale=totale+1;
            baricentro=baricentro+[M(i,j)*i,M(i,j)*j];
        end
    end
end
baricentro=baricentro/(totale);
baricentro=round(baricentro);

max_righe = 1;
max_colonne = 1;

for r = 1:Height
    cont = sum(mask(r,:));
    if(cont>max_righe)
        max_righe = cont;
    end
end

for c = 1:Width
    cont = sum(mask(:,c));
    if(cont>max_colonne)
        max_colonne = cont;
    end
end

if(max_righe>max_colonne)
    max_colonne = max_righe;
else
    max_righe = max_colonne;
end

coeff_altezza = (linspace(1,0,max_colonne+2))';
coeff_larghezza = linspace(1,0,max_righe+2);
alto_sx = conv2(coeff_altezza,coeff_larghezza);    
basso_sx = rot90(alto_sx);
basso_dx = rot90(basso_sx);    
alto_dx = rot90(basso_dx);

%% 

[max_colonne,max_righe] = size(alto_sx);

mask_a = zeros(size(mask));
mask_b = zeros(size(mask));
mask_c = zeros(size(mask));
mask_d = zeros(size(mask));

% Caso larghezza PARI | altezza PARI
if( mod(max_righe,2)==0 && mod(max_colonne,2)==0 )
    mask_a( baricentro(1)-floor(max_righe/2)+1 : baricentro(1)+floor(max_righe/2) , baricentro(2)-floor(max_colonne/2)+1 : baricentro(2)+floor(max_colonne/2) ) = alto_sx(:,:);
    mask_b( baricentro(1)-floor(max_righe/2)+1 : baricentro(1)+floor(max_righe/2) , baricentro(2)-floor(max_colonne/2)+1 : baricentro(2)+floor(max_colonne/2) ) = alto_dx(:,:);
    mask_c( baricentro(1)-floor(max_righe/2)+1 : baricentro(1)+floor(max_righe/2) , baricentro(2)-floor(max_colonne/2)+1 : baricentro(2)+floor(max_colonne/2) ) = basso_sx(:,:);
    mask_d( baricentro(1)-floor(max_righe/2)+1 : baricentro(1)+floor(max_righe/2) , baricentro(2)-floor(max_colonne/2)+1 : baricentro(2)+floor(max_colonne/2) ) = basso_dx(:,:);

% Caso larghezza PARI | altezza DISPARI
%elseif( mod(max_righe,2)==0 && mod(max_colonne,2)==1 )
%     mask_a( baricentro(1)-floor(max_righe/2)+1 : baricentro(1)+floor(max_righe/2) , baricentro(2)-floor(max_colonne/2) : baricentro(2)+floor(max_colonne/2) ) = alto_sx(:,:);
%     mask_b( baricentro(1)-floor(max_righe/2)+1 : baricentro(1)+floor(max_righe/2) , baricentro(2)-floor(max_colonne/2) : baricentro(2)+floor(max_colonne/2) ) = alto_dx(:,:);
%     mask_c( baricentro(1)-floor(max_righe/2)+1 : baricentro(1)+floor(max_righe/2) , baricentro(2)-floor(max_colonne/2) : baricentro(2)+floor(max_colonne/2) ) = basso_sx(:,:);
%     mask_d( baricentro(1)-floor(max_righe/2)+1 : baricentro(1)+floor(max_righe/2) , baricentro(2)-floor(max_colonne/2) : baricentro(2)+floor(max_colonne/2) ) = basso_dx(:,:);

% Caso larghezza DISPARI | altezza PARI
%elseif( mod(max_righe,2)==1 && mod(max_colonne,2)==0 )
%     mask_a( baricentro(1)-floor(max_righe/2) : baricentro(1)+floor(max_righe/2) , baricentro(2)-floor(max_colonne/2)+1 : baricentro(2)+floor(max_colonne/2) ) = alto_sx(:,:);
%     mask_b( baricentro(1)-floor(max_righe/2) : baricentro(1)+floor(max_righe/2) , baricentro(2)-floor(max_colonne/2)+1 : baricentro(2)+floor(max_colonne/2) ) = alto_dx(:,:);
%     mask_c( baricentro(1)-floor(max_righe/2) : baricentro(1)+floor(max_righe/2) , baricentro(2)-floor(max_colonne/2)+1 : baricentro(2)+floor(max_colonne/2) ) = basso_sx(:,:);
%     mask_d( baricentro(1)-floor(max_righe/2) : baricentro(1)+floor(max_righe/2) , baricentro(2)-floor(max_colonne/2)+1 : baricentro(2)+floor(max_colonne/2) ) = basso_dx(:,:);

% Caso larghezza DISPARI | altezza DISPARI
elseif( mod(max_righe,2)==1 && mod(max_colonne,2)==1 )
    mask_a( baricentro(1)-floor(max_righe/2) : baricentro(1)+floor(max_righe/2) , baricentro(2)-floor(max_colonne/2) : baricentro(2)+floor(max_colonne/2) ) = alto_sx(:,:);
    mask_b( baricentro(1)-floor(max_righe/2) : baricentro(1)+floor(max_righe/2) , baricentro(2)-floor(max_colonne/2) : baricentro(2)+floor(max_colonne/2) ) = alto_dx(:,:);
    mask_c( baricentro(1)-floor(max_righe/2) : baricentro(1)+floor(max_righe/2) , baricentro(2)-floor(max_colonne/2) : baricentro(2)+floor(max_colonne/2) ) = basso_sx(:,:);
    mask_d( baricentro(1)-floor(max_righe/2) : baricentro(1)+floor(max_righe/2) , baricentro(2)-floor(max_colonne/2) : baricentro(2)+floor(max_colonne/2) ) = basso_dx(:,:);

end

img_a = img_a.*mask_a;
img_b = img_b.*mask_b;
img_c = img_c.*mask_c;
img_d = img_d.*mask_d;

imgOut = img_a.*mask + img_b.*mask + img_c.*mask + img_d.*mask + I1;