function imgOut = LinPredVertPdim(I,mask,p)
% Metodo il quale, data un'image in ingresso, predice (solo) la zona selezionata
% dalla maschera.
% Predizione VERTICALE.
%
% X
% X
% X
% O
%           pi
% O
% X
% X
% X
%
% I = input intensity image
% p = predictor order
% img = output image
% Uses 1st-order optimal linear prediction to create the differential image

[Height,Width,Depth] = size(I);
if Depth > 1
    I1 = double(rgb2gray(I));
else
    I1 = double(I);
end

I1 = I1 .* (~mask);

img_alto = I1;
img_basso = I1;

% Coefficienti
alfa=zeros(1,p);
for i = 1:p
    alfa(i)=1/p;
end

for c = 1:Width
    
    %Devo contare quanti '1' ci sono nella colonna.
    contatore = 0;
    for r_cont = 1:Height
        if(mask(r_cont,c)>=1)
            contatore = contatore + 1;
        end
    end
    
    % Trovo i coefficienti
    if(contatore>0)
        coeff_alto = (linspace(1,0,contatore+2))';
        coeff_basso = 1-coeff_alto;
    end
    colonna_coeff_alto = zeros(Height,1);
    colonna_coeff_basso = zeros(Height,1);
    
    % Predizione
    j=1+1;
    
    for r = p+1:Height
        if(mask(r,c)>=1)
            temp_alto = 0;
            for i = 1:p
                temp_alto = temp_alto + alfa(i)*img_alto(r-i,c);
            end
            colonna_coeff_alto(r) = coeff_alto(j);
            j=j+1;
            img_alto(r,c) = img_alto(r,c) + temp_alto;
        end
    end    

    y=contatore+1;
       
    for r = Height-p:-1:1
        if(mask(r,c)>=1)
            temp_basso = 0;
            for i = 1:p
                temp_basso = temp_basso + alfa(i)*img_basso(r+i,c);
            end
            colonna_coeff_basso(r) = coeff_basso(y);
            y=y-1;
            img_basso(r,c) = img_basso(r,c) + temp_basso;
        end
    end    
    
    % Devo applicare i coefficienti
    img_alto(:,c) = img_alto(:,c).*colonna_coeff_alto;
    img_basso(:,c) = img_basso(:,c).*colonna_coeff_basso;
end
    
imgOut = img_alto.*mask + img_basso.*mask + I1;
    
    
    
    
    
    
    
    
    
    
    
    
    
    
